#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
All the CAL-ACCESS forms cataloged by our research.
"""
from calaccess_raw.annotations import FilingForm

all_filing_forms = (
    FilingForm(
        'F400',
        'Statement of Organization (Slate Mailer Organization)',
        group='CAMPAIGN',
        documentcloud_id='2781370-400-2016-01',
        description='Form 400 must be filed within 10 days after the slate \
mailer organization receives, or is promised to receive, $500 or more for \
producing one or more slate mailers.',
    ),
    FilingForm(
        'F401',
        'Campaign Disclosure Statement (Slate Mailer Organization)',
        group='CAMPAIGN',
        documentcloud_id='2781366-401-2005-01',
        description='Form 401 is filed by slate mailer organizations to \
disclose payments made and received in connection with producing slate mailers.',
    ),
    FilingForm(
        'F402',
        'Statement of Termination (Slate Mailer Organization)',
        group='CAMPAIGN',
        documentcloud_id='2781369-402-2005-01',
        description='Form 402 is filed by slate mailer organizations to \
terminate the organization.',
    ),
    FilingForm(
        'F405',
        'Amendment to Campaign Disclosure Statement',
        group='CAMPAIGN',
        documentcloud_id='2811582-405-1994',
        description="Form 405 is used to amend many campaign disclosure statements, with the exception of \
Statement of Organization (410), Candidate Intention (501) or Campaign Bank Account (502) forms.",
    ),
    FilingForm(
        'F410',
        'Statement of Organization (Recipient Committee)',
        group='CAMPAIGN',
        documentcloud_id='2781368-410-2016-01',
        description='Form 410 must be filed within 10 days of receiving \
$2,000 in contributions. If the committee has not yet reached the $2,000 \
threshold, the not-yet-qualified box should be checked.',
    ),
    FilingForm(
        'F415',
        'Title Unknown',
        group='Deprecated',
        description='Form 415 was deprecated in or around 2001. The information \
previously reported on this form is now reported on Form 410 (Statement of \
Organization Recipient Committee). There are 51,047 filings with this form ID.',
    ),
    FilingForm(
        'F416',
        'Title Unknown',
        group='Deprecated',
        description='Form 416 was deprecated in or around 2001. The information \
previously reported on this form is now reported on Form 410 (Statement of \
Organization Recipient Committee). There are 521 filings with this form ID.',
    ),
    FilingForm(
        'F419',
        'Campaign Disclosure Statement, Long Form (Ballot Measure Committee)',
        group='Deprecated',
        description='Form 419 was replaced by Form 460 (Recipient Committee \
Campaign Statement) in or around 2001. There are 2,293 filings with this form ID.',
    ),
    FilingForm(
        'F420',
        'Campaign Disclosure Statement, Long Form (Recipient Committee)',
        group='Deprecated',
        description='Form 420 was replaced by Form 460 (Recipient Committee \
Campaign Statement) in or around 2001. There are 70,704 filings with this form ID.',
    ),
    FilingForm(
        'F425',
        'Semi-Annual Statement of No Activity (Recipient Committee)',
        group='CAMPAIGN',
        documentcloud_id='2781365-425-2001-01',
        description='Form 425 is filed by recipient committees that have not \
received any contributions and have not made any expenditures during the six-\
month period covered by a semi-annual statement.',
    ),
    FilingForm(
        'F430',
        'Title Unknown',
        group='Deprecated',
        description='Form 430 was deprecated in or around 1980. The information \
previously reported on this form is (probably) now reported on Form 460 (Recipient \
Committee Campaign Statement). There are 2,293 filings with this form ID.',
    ),
    FilingForm(
        'F450',
        'Campaign Disclosure Statement, Short Form (Recipient Committee)',
        group='CAMPAIGN',
        documentcloud_id='2781364-450-2016-01',
        description='Form 450 is filed by recipient committees that meet \
certain specific criteria listed in the Form 450.',
    ),
    FilingForm(
        'F460',
        'Campaign Disclosure Statement (Recipient Committee)',
        group='CAMPAIGN',
        documentcloud_id='2781363-460-2016-01',
        description='Form 460 is filed by recipient committees to report \
expenditures and contributions. It can be used to file a pre-election statement, \
semi-annual statement, quarterly statement, termination statement, special odd-\
year report or an amendment to a previously filed statement.',
    ),
    FilingForm(
        'F461',
        'Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)',
        group='CAMPAIGN',
        documentcloud_id='2781361-461-2016-01',
        description='Form 461 is filed by major donors, independent \
expenditure committees and multipurpose organizations including nonprofits.',
    ),
    FilingForm(
        'F465',
        'Supplemental Independent Expenditure Report',
        group='CAMPAIGN',
        documentcloud_id='2781358-465-2009-06',
        description='Form 465 is filed by officeholders, candidates, recipient \
committees, major donor committees and independent expenditure committees that \
make independent expenditures totaling $1,000 or more in a calendar year to support \
or oppose a single candidate, a single measure, or the qualification of a single \
measure. Filed in the same periods the candidate or committee \
supported or opposed by the independent expenditures is required to file.',
    ),
    FilingForm(
        'F470',
        'Campaign Disclosure Statement, Short Form (Officeholders and Candidates)',
        group='CAMPAIGN',
        documentcloud_id='2781357-470-2016-01',
        description='Form 470 is filed by officeholders and candidates who do \
not have a controlled committee, do not spend or receive contributions totaling $2,000 or \
more during the calendar year.'),
    FilingForm(
        'F490',
        'Campaign Disclosure Statement, Long Form (Officeholders and Candidates)',
        group='Deprecated',
        description='Form 490 was replaced by Form 460 in or around 2001. There \
are 58,266 filings with this form ID.',
    ),
    FilingForm(
        'F495',
        'Supplemental Pre-Election Campaign Statement (Recipient Committee)',
        group='CAMPAIGN',
        documentcloud_id='2781356-495-2005-01',
        description='Form 495 is filed by recipient committees that make \
contributions totaling $10,000 or more in connection with an election in which \
the committee is not required to file regular preelection reports. Form 495 is \
filed as an attachment to Form 450 or Form 460.'
    ),
    FilingForm(
        'F496',
        'Late Independent Expenditure Report',
        group='CAMPAIGN',
        documentcloud_id='2781355-496-2016-01',
        description='Form 496 is filed by committees that make independent \
expenditures whose combined total is $1,000 or more to support or oppose a single \
candidate for office or a single ballot measure. Form 496 should be \
filed within 24-hours of making the expenditure during the 90 days immediately \
preceding the election.',
    ),
    FilingForm(
        'F497',
        'Late Contribution Report',
        group='CAMPAIGN',
        documentcloud_id='2781353-497-2016-01',
        description='Form 497 is filed by state and local committees making \
or receiving contributions totaling $1,000 or more in the 90 \
days before an election, committees reporting contributions of $5,000 or more in \
connection with a state ballot measure and state candidates as well as state \
ballot measure committees that receive $5,000 or more at any time other than a \
90-day election cycle.',
    ),
    FilingForm(
        'F498',
        'Late Payment Report (Slate Mailer Organization)',
        group='CAMPAIGN',
        documentcloud_id='2781352-498-2016-01',
        description='Form 498 is filed by a slate mailer organization on \
receipt of a late payment.',
    ),
    FilingForm(
        'F501',
        'Candidate Intention Statement',
        group='CAMPAIGN',
        documentcloud_id='2781351-501-2016-01',
        description='Form 501 is filed each election by candidates for state \
or local office.'
    ),
    FilingForm(
        'F502',
        'Campaign Bank Account Statement',
        group='CAMPAIGN',
        description='Form 502 must be filed within 10 days of opening a \
campaign bank account at a financial institution in California.'
    ),
    FilingForm(
        'F511',
        'Paid Spokesperson Report',
        group='CAMPAIGN',
        documentcloud_id='2781350-511-2015-01',
        description='Form 511 is filed by committees that make expenditures \
totaling $5,000 or more to an individual for his or her appearance in a printed, \
televised, or radio advertisement, or in a telephone message to support or \
oppose the qualification, passage, or defeat of a state or local ballot measure.'
    ),
    FilingForm(
        'E530',
        'Electronic Issue Advocacy Report',
        group='CAMPAIGN',
        documentcloud_id='2781349-E530-Instructions',
        description='On-line Form E-530 reports must be filed by anyone \
spending or promising to pay $50,000 or more for a communication disseminated \
within 45 days of an election, if the communication clearly identifies a candidate \
for state elective office but does not expressly advocate the election or defeat \
of that candidate.'
    ),
    FilingForm(
        'F601',
        'Lobbying Firm Registration Statement',
        group='LOBBYIST',
        documentcloud_id='2781348-601-2014-10',
        description='Form 601 is filed on a biennial basis by a lobbying \
firm or individual contract lobbyist wishing to register or renew an existing \
registration. The form must be filed within 10 days of qualifying as a lobbying \
firm. Renewal of existing registration is due between November 1 and December 31 \
of each even-numbered year. This registration is valid for the complete two-year \
cycle of such session.'),
    FilingForm(
        'F602',
        'Lobbying Firm Activity Authorization',
        group='LOBBYIST',
        documentcloud_id='2781347-602-1998-07',
        description='Form 602 is an authorization form filed by each person \
who employs or contracts with a lobbying firm. This form serves as an attachment \
to Form 601, and is filed by the applicable lobbying firm. Form 602 also contains \
a schedule which describes by category the nature and interest of the client of \
the firm. Like Form 601 this registration attachment is valid for the length of \
the state legislative session for which it is filed. Form 602 must be filed by \
a firm or its client, prior to attempting to influence legislative or \
administrative action on behalf of that client.'
    ),
    FilingForm(
        'F603',
        'Lobbyist Employer or Lobbying Coalition Registration Statement',
        group='LOBBYIST',
        documentcloud_id='2781346-603-2014-10',
        description='Form 603 is a registration statement filed by registered \
lobbyists employers or lobbying coalitions upon qualifying as an employer or \
coalition. This form is also used to renew an existing registration on a biennial \
basis. Form 603 must be filed within 10 days of qualifying as a lobbyist employer \
or lobbying coalition. Renewal of an existing registration is due between November \
1 and December 31 of each even-numbered year. This registration is valid for the \
complete two-year cycle of such session.'
    ),
    FilingForm(
        'F604',
        'Lobbyist Certification Statement',
        group='LOBBYIST',
        documentcloud_id='2781345-604-2014-10',
        description="Form 604 is the certification statement filed by an \
individual who qualifies as a lobbyist (including an individual contract lobbyist\
). Form 604 is the initial certification statement and is also used as a renewal \
of a previous lobbyist certification. This form includes verification as to \
whether the lobbyist has attended a required course within the previous 12 months \
on ethical issues and laws relating to lobbying. When submitted as a paper filing\
, this form is an attachment to either the firm's Form 601 or the employer's Form \
603. If the form is filed electronically, it is filed separately by the lobbyist."
    ),
    FilingForm(
        'F605',
        'Amendment to Registration, Lobbying Firm, Lobbyist Employer, Lobbying Coalition',
        group='LOBBYIST',
        documentcloud_id='2781344-605-2014-10',
        description='Form 605 is the standard amendment form used to amend \
any previously filed registration information. It is used to add or delete both \
lobbyists and clients to an existing registration. It is also used to change name\
, address, and responsible officer information, as well as any other pertinent \
information found on forms 601, 602, 603 or 604.'
    ),
    FilingForm(
        'F606',
        'Notice of Termination',
        group='LOBBYIST',
        documentcloud_id='2781343-606-1997',
        description='Form 606 is filed by any lobbying firm, registered \
lobbyist employer, lobbying coalition or lobbyist who wishes to terminate a filed \
registration or certification statement. A client of a firm (non-registered employer\
) does not use this form to cease lobbying activity. Instead it is deleted by the \
associated firm, which files a Form 605. Form 606 is filed within 20 days of \
ceasing all lobbying activity. A final quarterly disclosure statement must be filed \
for the quarter in which the date of termination is effective.'
    ),
    FilingForm(
        'F607',
        'Notice of Withdrawal',
        group='LOBBYIST',
        documentcloud_id='2781342-607-1997-08',
        description='Form 607 is filed by a lobbying firm or lobbyist wishing \
to withdraw the filed registration statement of a firm which has never met the \
statutory definition of a lobbying firm or lobbyist. Submittal of this form \
relieves the filer of any duty to file any previously-required quarterly \
disclosure statements.'
    ),
    FilingForm(
        'F615',
        'Lobbyist Report',
        group='LOBBYIST',
        documentcloud_id='2781341-615-1990',
        description='Form 615 is the quarterly disclosure statement completed \
by the in-house lobbyist of a lobbying firm, lobbyist employer, or lobbying \
coalition. It is not filed on its own. For paper filers, it is an \
attachment to either Form 625 (Report of Lobbying Firm) or Form 635. (Report of \
Lobbyist Employer/Lobbying Coalition). Electronic or online filers file these as \
separate documents.'
    ),
    FilingForm(
        'F625',
        'Report of Lobbying Firm',
        group='LOBBYIST',
        documentcloud_id='2781340-625-1990',
        description='Form 625 is the quarterly disclosure statement filed by \
a lobbying firm (including individual contract lobbyists) each calendar quarter. \
If the firm employs one or more in-house lobbyists, then, for paper filers, a \
separate Form 615 (Lobbyist Report) must be attached for each lobbyist. Electronic \
or online filers file these as separate documents.'
    ),
    FilingForm(
        'S630',
        'Payments Made to Lobbying Coalitions (Attachment to Form 625 or 635) ',
        group='LOBBYIST',
        documentcloud_id='2782806-630-1990',
        description='An attachment to the quarterly disclosure report filed \
by a lobbying firm or lobbyist employer which makes payments to a lobbying \
coalition. This attachment itemizes such payments.'
    ),
    FilingForm(
        'F635',
        'Report of Lobbyist Employer or Report of Lobbying Coalition',
        group='LOBBYIST',
        documentcloud_id='2781339-635-1993',
        description='Form 635 is the quarterly disclosure statement filed by \
a lobbyist employer or a lobbying coalition. For employers and lobbying \
coalitions filing on paper, a separate Form 615 must be completed for each in-\
house lobbyist and attached to Form 635. Electronic or online filers file these \
as separate documents. This form is also used as a quarterly disclosure statement \
for a client of a firm which has no in-house lobbyist (also referred to as a \
non-registered employer).'
    ),
    FilingForm(
        'S635C',
        'Payments Received by Lobbying Coalitions',
        db_value='S635-C',
        group='LOBBYIST',
        documentcloud_id='2781338-635C-1990',
        description='Form 635C is filed by a lobbying coalition as an \
attachment to the Form 635 (Report of a Lobbying Coalition) and discloses all \
payment received from the members of a coalition.'
    ),
    FilingForm(
        'S640',
        'Governmental Agencies Reporting (Attachment to Form 635 or Form 645)',
        group='LOBBYIST',
        documentcloud_id='2781337-640-1993',
        description='Form 640 is filed by a state or local governmental agency \
which qualifies as a lobbyist employer or $5,000 filer. The attachment replaces \
Section D of Form 635 and Section B of Form 645 (Other Payments to \
Influence Legislative or Administrative Action). It is filed in conjunction with \
either Form 635 if a lobbyist employer or Form 645 if a $5,000 filer.'
    ),
    FilingForm(
        'F645',
        'Report of Person Spending $5,000 or More',
        group='LOBBYIST',
        documentcloud_id='2781336-645-1993',
        description='Form 645 is the quarterly disclosure document filed by a \
$5,000 filer (i.e. a person who does not employ a lobbyist or contract with a lobbying \
firm, but who makes payments to influence legislative or administrative action \
in aggregation of $5,000 or more in any calendar quarter). The filer does not \
submit a registration or termination statement, and is only required to file Form \
645 in those calendar quarters which $5,000 or more is spent to influence legislative \
or administrative action. Form 645 must be filed electronically.'
    ),
    FilingForm(
        'F690',
        'Amendment to Lobbying Disclosure Report',
        group='LOBBYIST',
        documentcloud_id='2781335-690-1990',
        description='Form 690 is filed by a lobbying firm, lobbyist employer, \
lobbying coalition, $5,000 filer or lobbyist seeking to amend any information \
previously submitted on a quarterly disclosure report. Any amendment to the \
registration statement should be made on Form 605 rather than Form 690. \
Amendments must be filed by the same method (paper or electronic) as the original \
form.'
    ),
    FilingForm(
        'F700',
        'Statement of Economic Interest',
        group='FINANCIAL DISCLOSURE',
        documentcloud_id='2792958-700-2015-12',
        description='Every public official who makes or participates in making \
governmental decisions is required to file.',
    ),
    FilingForm(
        'F900',
        'Campaign Disclosure Statement (Public employee retirement board candidate)',
        group='CAMPAIGN',
    ),
)


def get_filing_form(id):
    """
    Takes an id for a filing form and returns a FilingForm object.
    """
    return next((x for x in all_filing_forms if x.id == id.upper()), None)


# adding Filing Form Sections

form = get_filing_form('F400')
form.add_section(
    id='P1',
    title='Part 1, Slate Mailer Organization Information',
    start_page=2,
)
form.add_section(
    id='P2',
    title='Part 2, Treasurer And Other Principal Officers',
    start_page=2,
)
form.add_section(
    id='P3',
    title='Part 3, Individuals Who Authorize Contents Of Slate Mailers',
    start_page=3,
)
form.add_section(
    id='P4',
    title='Part 4, Is This Organization A "Committee" Pursuant To Government Code Section 82013?',
    start_page=3,
)
form.add_section(
    id='P5',
    title='Part 5, Verification',
    start_page=3,
)

form = get_filing_form('F401')
form.add_section(
    id='CVR',
    title='Cover Page',
    start_page=3,
    end_page=4,
)
form.add_section(
    id='A',
    db_value='F401A',
    title='Schedule A, Payments Received',
    start_page=5,
    end_page=7,
)
form.add_section(
    id='B',
    db_value='F401B',
    title='Schedule B, Payments Made',
    start_page=8,
    end_page=9,
)
form.add_section(
    id='B-1',
    db_value='F401B-1',
    title='Schedule B-1, Payments Made by Agent or Independent Contractor',
    start_page=10,
)
form.add_section(
    id='C',
    db_value='F401C',
    title='Schedule C, Persons Receiving $1,000 or More',
    start_page=11,
    end_page=12,
)
form.add_section(
    id='D',
    db_value='F401D',
    title='Schedule D, Candidates and Measures Not Listed on Schedule A',
    start_page=13,
    end_page=14,
)

form = get_filing_form('F402')
form.add_section(
    id='CVR',
    title='Cover Page',
    start_page=2,
)
form.add_section(
    id='VER',
    title='Verification',
    start_page=2,
)

form = get_filing_form('F410')
form.add_section(
    id='P1',
    title='Part 1, Committee Information',
    start_page=2,
)
form.add_section(
    id='P2',
    title='Part 2, Treasurer and Other Principal Officers',
    start_page=2,
)
form.add_section(
    id='P3',
    title='Part 3, Verification',
    start_page=2,
)
form.add_section(
    id='P4',
    title='Part 4, Type of Committee',
    start_page=2,
    end_page=3,
)

form = get_filing_form('F425')
form.add_section(
    id='P1',
    title='Part 1, Committee Information',
    start_page=1,
)
form.add_section(
    id='P2',
    title='Part 2, Period of No Activity',
    start_page=1,
)
form.add_section(
    id='P3',
    title='Part 3, Verification',
    start_page=1,
)

form = get_filing_form('F450')
form.add_section(
    id='CVR',
    title='Cover Page, Type of Recipient Committee',
    start_page=3
)
form.add_section(
    id='P1',
    title='Part 1, Payments Made',
    start_page=3
)
form.add_section(
    id='P2',
    title='Part 2, Type of Statement',
    start_page=3
)
form.add_section(
    id='P3',
    title='Part 3, Committee Information',
    start_page=3
)
form.add_section(
    id='P4',
    title='Part 4, Verification',
    start_page=3
)
form.add_section(
    id='SMRY',
    title='Summary Page',
    start_page=5
)
form.add_section(
    id='P5',
    db_value='F450P5',
    title='Part 5, Payments Made',
    start_page=6,
    end_page=7,
)

form = get_filing_form('F460')
form.add_section(
    id='CVR',
    title='Cover Page, Part 1',
    start_page=3,
    end_page=4,
    db_value='CVR',
)
form.add_section(
    id='CVR2',
    title='Cover Page, Part 2',
    start_page=2,
    db_value='CVR2',
)
form.add_section(
    id='SMRY',
    title='Summary Page',
    start_page=7,
    end_page=8,
    db_value='SMRY',
)
form.add_section(
    id='A',
    title='Schedule A, Monetary Contributions Received',
    start_page=9,
    end_page=11,
    db_value='A',
)
# this section is not in the sample form downloaded from the FPPC site
# but does appear in the filings
form.add_section(
    id='A-1',
    title='Schedule A-1, Contributions Transferred to Special Election Commitee',
    db_value='A-1',
)
form.add_section(
    id='B1',
    title='Schedule B, Part 1, Loans Received',
    start_page=12,
    end_page=13,
    db_value='B1',
)
form.add_section(
    id='B2',
    title='Schedule B, Part 2, Loan Guarantors',
    start_page=14,
    end_page=15,
    db_value='B2',
)
# this section appears in older versions of Form 460, but not in the current one
form.add_section(
    id='B3',
    title='Schedule B, Part 3, Outstanding Balance',
    db_value='B3',
)
form.add_section(
    id='C',
    title='Schedule C, Non-Monetary Contributions Received',
    start_page=16,
    end_page=17,
    db_value='C',
)
form.add_section(
    id='D',
    title='Schedule D, Summary of Expenditures Supporting / Opposing Other \
Candidates, Measures and Committees',
    start_page=18,
    end_page=20,
    db_value='D',
)
form.add_section(
    id='E',
    title='Schedule E, Payments Made',
    start_page=21,
    end_page=24,
    db_value='E',
)
form.add_section(
    id='F',
    title='Schedule F, Accrued Expenses (Unpaid Bills)',
    start_page=25,
    end_page=27,
    db_value='F',
)
form.add_section(
    id='G',
    title='Schedule G, Payments Made by an Agent or Independent Contractor \
(on Behalf of This Committee)',
    start_page=28,
    end_page=29,
    db_value='G',
)
form.add_section(
    id='H',
    title='Schedule H, Loans Made to Others',
    start_page=29,
    end_page=30,
    db_value='H',
)
# this section is not in the sample form downloaded from the FPPC site
# but does appear in the filings
form.add_section(
    id='H1',
    title='Schedule H, Part 1, Loans Made',
    db_value='H1',
)
# this section is not in the sample form downloaded from the FPPC site
# but does appear in the filings
form.add_section(
    id='H2',
    title='Schedule H, Part 2, Repayments Rcvd',
    db_value='H2',
)
# this section is not in the sample form downloaded from the FPPC site
# but does appear in the filings
form.add_section(
    id='H3',
    title='Schedule H, Part 3, Outstanding Loans',
    db_value='H3',
)
form.add_section(
    id='I',
    title='Schedule I, Miscellanous increases to cash',
    start_page=31,
    end_page=32,
    db_value='I',
)

form = get_filing_form('F461')
form.add_section(
    id='P1',
    title='Part 1, Name and Address of Filer',
    start_page=3,
    db_value='F461P1',
)
form.add_section(
    id='P2',
    title='Part 2, Nature and Interests of Filer',
    start_page=3,
    db_value='F461P2',
)
form.add_section(
    id='P3',
    title='Part 3, Summary',
    start_page=3,
    db_value='F461P3',
)
form.add_section(
    id='P4',
    title='Part 4, Verification',
    start_page=3,
    db_value='F461P4',
)
form.add_section(
    id='P5',
    title='Part 5, Contributions (Including Loans, Forgiveness of Loans, and Loan\
Guarantees) and Expenditures Made',
    start_page=5,
    end_page=6,
    db_value='F461P5',
)

form = get_filing_form('F465')
form.add_section(
    id='P1',
    title='Part 1, Committee/Filer Information',
    start_page=2,
    db_value='F465P1',
)
form.add_section(
    id='P2',
    title='Part 2, Name of Candidate or Measure Supported or Opposed',
    start_page=2,
    db_value='F465P2',
)
form.add_section(
    id='P3',
    title='Part 3, Independent Expenditures Made',
    start_page=2,
    db_value='F465P3',
)
form.add_section(
    id='P4',
    title='Part 4, Summary',
    start_page=4,
    db_value='F465P4',
)
form.add_section(
    id='P5',
    title='Part 5, Filing Officers',
    start_page=4,
    db_value='F465P5',
)
form.add_section(
    id='P6',
    title='Part 6, Verification',
    start_page=4,
    db_value='F465P6',
)

form = get_filing_form('F496')
form.add_section(
    id='P1',
    title='Part 1, List Only One Candidate or Ballot Measure',
    start_page=3,
    db_value='F496P1',
)
form.add_section(
    id='P2',
    title='Part 2, Independent Expenditures Made',
    start_page=3,
    db_value='F496P2',
)
form.add_section(
    id='P3',
    title='Part 3, Contributions > $100 Received',
    start_page=3,
    db_value='F496P3',
)

form = get_filing_form('F497')
form.add_section(
    id='P1',
    title='Part 1, Contributions Received',
    start_page=2,
    db_value='F497P1',
)
form.add_section(
    id='P2',
    title='Part 2, Contributions Made',
    start_page=4,
    db_value='F497P2',
)

form = get_filing_form('F498')
form.add_section(
    id='A',
    title='Part A, Late Payments Attributed To',
    db_value='F498-A',
)
form.add_section(
    id='R',
    title='Part R, Late Payments Received From',
    start_page=2,
    db_value='F498-R',
)

form = get_filing_form('F601')
form.add_section(
    id='P1',
    db_value='F601P1',
    title='Part 1, Individual Lobbyists',
    start_page=2,
)
form.add_section(
    id='P2A',
    db_value='F601P2A',
    title='Part 2, Section A, Lobbyist Employers',
    start_page=2,
    end_page=4,
)
form.add_section(
    id='P2B',
    db_value='F601P2B',
    title='Part 2, Section B, Subcontracted Clients',
    start_page=4,
)

form = get_filing_form('F615')
form.add_section(
    id='P1',
    db_value='F615P1',
    title='Part 1, Activity Expenses Paid, Incurred, Arranged or Provided by the Lobbyist',
    start_page=2,
    end_page=4,
)
form.add_section(
    id='P2',
    db_value='F615P2',
    title='Part 2, Campaign Contributions Made or Delivered',
    start_page=5,
)

form = get_filing_form('F625')
form.add_section(
    id='P1',
    db_value='F625P1',
    title='Part 1, Partners, Owners, Officers, and Employees',
    start_page=2,
)
form.add_section(
    id='P2',
    db_value='F625P2',
    title='Part 2, Payments Received in Connection with Lobbying Activity',
    start_page=4,
)
form.add_section(
    id='P3A',
    db_value='F625P3A',
    title='Part 3, Payments Made In Connection With Lobbying Activities, \
Section A, Activity Expenses',
    start_page=4,
)
form.add_section(
    id='P3B',
    db_value='F625P3B',
    title='Part 3, Payments Made In Connection With Lobbying Activities, \
Section B, Payments Made',
    start_page=8,
)
form.add_section(
    id='P3C',
    db_value='F625P3C',
    title='Part 3, Payments Made In Connection With Lobbying Activities, \
Section C, Summary of Payments',
    start_page=2,
)
form.add_section(
    id='P4B',
    db_value='F625P4B',
    title='Part 4, Campaign Contributions Made',
    start_page=2,
)

form = get_filing_form('F635')
form.add_section(
    id='P1',
    db_value='F635P1',
    title='Part 1, Legislative or State Agency Administrative Actions Actively \
Lobbied During the Period',
    start_page=2,
)
form.add_section(
    id='P2',
    db_value='F635P2',
    title='Part 2, Partners, Owners, and Employees whose "Lobbyist Reports" \
(Form 615) are Attached to this Report',
    start_page=4,
)
form.add_section(
    id='P3A',
    db_value='F635P3A',
    title='Part 3, Payments Made in Connection with Lobbying Activities, \
Section A, Payments To In-house Employee Lobbyists',
    start_page=4,
)
form.add_section(
    id='P3B',
    db_value='F635P3B',
    title='Part 3, Payments Made in Connection with Lobbying Activities, \
Section B, Payments To Lobbying Firms',
    start_page=4,
)
form.add_section(
    id='P3C',
    db_value='F635P3C',
    title='Part 3, Payments Made in Connection with Lobbying Activities, \
Section C, Activity Expenses',
    start_page=6,
)
form.add_section(
    id='P3D',
    db_value='F635P3D',
    title='Part 3, Payments Made in Connection with Lobbying Activities, \
Section D, Other Payments to Influence Legislative or Administrative Action',
    start_page=6,
)
form.add_section(
    id='P3E',
    db_value='F635P3E',
    title='Part 3, Payments Made in Connection with Lobbying Activities, \
Section E, Payments in Connection with Administrative Testimony in Ratemaking \
Proceedings Before The California Public Utilities Commission',
    start_page=6,
)
form.add_section(
    id='P4B',
    db_value='F635P4B',
    title='Part 4, Campaign Contributions Made',
    start_page=8,
)

form = get_filing_form('F645')
form.add_section(
    id='P1',
    db_value='F645P1',
    title='Part 1, Legislative or State Agency Administrative Actions Actively \
Lobbied during the Period',
    start_page=2,
)
form.add_section(
    id='P2A',
    db_value='F645P2A',
    title='Part 2, Payments Made this Period, Section A, Activity Expenses',
    start_page=4,
)
form.add_section(
    id='P2B',
    db_value='F645P2B',
    title='Part 2, Payments Made this Period, Section B, Other Payments to \
Influence Legislative or Administrative Action',
    start_page=4,
)
form.add_section(
    id='P2C',
    db_value='F645P2C',
    title='Part 2, Payments Made this Period, Section C, Payments in Connection \
with Administrative Testimony in Ratemaking Proceedings Before the California \
Public Utilities Commission',
    start_page=4,
)
form.add_section(
    id='P3B',
    db_value='F645P3B',
    title='Part 3, Campaign Contributions Made',
    start_page=4,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom managers for working with CAL-ACCESS data.
"""
from __future__ import unicode_literals
import os
from django.db import models, connection
from calaccess_raw import get_download_directory


class CalAccessManager(models.Manager):
    """
    Utilities for accessing the raw data associated with a model.
    """
    def get_csv_name(self):
        """
        Returns the name of the model's CSV data file.
        """
        return "%s.csv" % self.model._meta.db_table.lower()

    def get_csv_path(self):
        """
        Returns the path to the model's CSV data file.
        """
        return os.path.join(
            get_download_directory(),
            'csv',
            self.get_csv_name()
        )

    def get_tsv_name(self):
        """
        Returns the name of the model's raw TSV data file.
        """
        return "%s.TSV" % self.model._meta.db_table

    def get_tsv_path(self):
        """
        Returns the path to the model's raw TSV data file.
        """
        return os.path.join(
            get_download_directory(),
            'tsv',
            self.get_tsv_name()
        )

    def add_constraints_and_indexes(self):
        """
        Re-create constraints and indexes on the model and its fields.
        """
        with connection.schema_editor() as schema_editor:
            schema_editor.alter_unique_together(
                self.model,
                (),
                self.model._meta.unique_together,
            )

            schema_editor.alter_index_together(
                self.model,
                (),
                self.model._meta.index_together,
            )

            for field in self.model.objects.constrained_fields:
                field_copy = field.__copy__()
                field_copy.db_constraint = False
                schema_editor.alter_field(
                    self.model, field_copy, field
                )

            for field in self.model.objects.indexed_fields:
                field_copy = field.__copy__()
                field_copy.db_index = False
                schema_editor.alter_field(
                    self.model, field_copy, field
                )

    def drop_constraints_and_indexes(self):
        """
        Temporarily drop constraints and indexes on the model and its fields.
        """
        with connection.schema_editor() as schema_editor:
            schema_editor.alter_unique_together(
                self.model,
                self.model._meta.unique_together,
                (),
            )

            schema_editor.alter_index_together(
                self.model,
                self.model._meta.index_together,
                (),
            )

            for field in self.model.objects.constrained_fields:
                field_copy = field.__copy__()
                field_copy.db_constraint = False
                schema_editor.alter_field(
                    self.model, field, field_copy
                )

            for field in self.model.objects.indexed_fields:
                field_copy = field.__copy__()
                field_copy.db_index = False
                schema_editor.alter_field(
                    self.model, field, field_copy
                )

    @property
    def constrained_fields(self):
        """
        Returns list of model's fields with db_constraint set to True.
        """
        return [
            f for f in self.model._meta.fields
            if hasattr(f, 'db_constraint') and f.db_constraint
        ]

    @property
    def indexed_fields(self):
        """
        Returns list of model's fields with db_index set to True.
        """
        return [
            f for f in self.model._meta.fields if f.db_index
        ]


class RawDataVersionQuerySet(models.QuerySet):
    """
    Custom methods for working with a RawDataVersion QuerySet.
    """
    def complete(self):
        """
        Filters down QuerySet to return only version that have a complete update.
        """
        return self.exclude(update_finish_datetime__isnull=True)


class RawDataVersionManager(models.Manager):
    """
    Custom methods for working with the RawDataVersion model.
    """
    def get_queryset(self):
        """
        Returns the custom QuerySet we want for this manager.
        """
        return RawDataVersionQuerySet(self.model, using=self._db)

    def complete(self):
        """
        Filters down QuerySet to return only version that have a complete update.
        """
        return self.get_queryset().complete()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Download, unzip, clean and load the latest CAL-ACCESS database ZIP.
"""
import os
import logging
from sys import exit
from time import sleep
from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile
from django.core.files import File
from django.conf import settings
from django.contrib.humanize.templatetags.humanize import naturaltime
from clint.textui import progress
from django.utils.timezone import now
from django.core.management import call_command
from calaccess_raw.management import handle_command
from django.core.management.base import CommandError
from django.template.loader import render_to_string
from calaccess_raw.models.tracking import RawDataVersion
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.management.commands.extractcalaccessrawfiles import TestCommand as TestExtractCommand
from calaccess_raw import (
    get_download_directory,
    get_test_download_directory,
    get_model_list
)
logger = logging.getLogger(__name__)


class Command(CalAccessCommand):
    """
    Download, unzip, clean and load the latest CAL-ACCESS database ZIP.
    """
    help = "Download, unzip, clean and load the latest CAL-ACCESS database ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep zip, unzipped, TSV and CSV files"
        )
        parser.add_argument(
            "--noinput",
            action="store_true",
            dest="noinput",
            default=False,
            help="Update or resume previous update without asking permission"
        )
        parser.add_argument(
            "--test",
            "--use-test-data",
            action="store_true",
            dest="test_data",
            default=False,
            help="Use sampled test data (skips download, clean a load)"
        )
        parser.add_argument(
            "-a",
            "--app-name",
            dest="app_name",
            default="calaccess_raw",
            help="Name of Django app with models into which data will "
                 "be imported (if other not calaccess_raw)"
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # set / compute any attributes that multiple class methods need
        self.app_name = options["app_name"]
        self.keep_files = options["keep_files"]
        self.test_mode = options['test_data']
        self.noinput = options['noinput']

        if self.test_mode:
            # and always keep files when running test data
            self.keep_files = True
            self.data_dir = get_test_download_directory()
            # need to set this app-wide because cleancalaccessrawfile
            #   also calls get_download_directory
            settings.CALACCESS_DOWNLOAD_DIR = self.data_dir
            self.noinput = True
        else:
            self.data_dir = get_download_directory()

        # set the download zip file path
        self.zip_path = os.path.join(self.data_dir, self.url.split('/')[-1])
        # make the data dir if necessary
        os.path.exists(self.data_dir) or os.makedirs(self.data_dir)
        self.tsv_dir = os.path.join(self.data_dir, "tsv/")
        # Immediately check that the tsv directory exists when using test data,
        #   so we can stop immediately.
        if self.test_mode:
            if not os.path.exists(self.tsv_dir):
                raise CommandError("Data tsv directory does not exist "
                                   "at %s" % self.tsv_dir)
            elif self.verbosity:
                self.log("Using test data")

        # set up the dir for clean files
        self.csv_dir = os.path.join(self.data_dir, "csv/")
        os.path.exists(self.csv_dir) or os.makedirs(self.csv_dir)

        # in test mode, get download metadata from local file
        if self.test_mode:
            with open(self.data_dir + "/sampled_version.txt", "r") as f:
                # read expected_size from sampled_version.txt
                try:
                    # long int type is big enough for double the current size of the zip
                    expected_size = long(f.readline())
                except NameError:
                    # in py3, no long(), instead int will suffice
                    expected_size = int(f.readline())
                # and release_datetime
                release_datetime = self.parse_imf_datetime_str(f.readline())
                # get or create the RawDataVersion
                latest_version, created = self.get_or_create_version(
                    expected_size,
                    release_datetime,
                )
        # else request it
        else:
            download_metadata = self.get_download_metadata()
            logger.debug('Server: %s' % download_metadata['server'])
            logger.debug('ETag: %s' % download_metadata['etag'])
            logger.debug('Last-Modified: %s' % download_metadata['last-modified'])
            logger.debug('Content-Length: %s' % download_metadata['content-length'])
            # get or create the RawDataVersion
            latest_version, created = self.get_or_create_version(
                download_metadata['content-length'],
                self.parse_imf_datetime_str(
                    download_metadata['last-modified']
                ),
            )
        # log if latest version is new
        if created:
            logger.info('New CAL-ACCESS version available.')

        previous_version = None
        since_previous_release = None
        can_resume_previous = False
        # if update to latest version has not yet started, check to see if previous
        #   version can resume
        if not latest_version.update_start_datetime:
            # get previous versions
            prev_downloaded_versions = RawDataVersion.objects.filter(
                download_start_datetime__isnull=False
            ).exclude(id=latest_version.id)
            # if there are any
            if prev_downloaded_versions:
                # get the last one started
                previous_version = prev_downloaded_versions.latest(
                    'download_start_datetime'
                )
                # for prompt, get time since previous release
                since_previous_release = naturaltime(
                    previous_version.release_datetime
                )
                # if the previous version finished downloading and didn't finish updating
                if (
                    previous_version.download_completed and
                    not previous_version.update_completed
                ):
                    # can resume if zip file has not been deleted
                    #   or if the raw files have been extracted
                    if (
                        os.path.exists(self.zip_path) or
                        previous_version.extract_completed
                    ):
                        can_resume_previous = True

        # if not accepting input
        if self.noinput:
            # skip if up-to-date
            if latest_version.update_completed:
                self.log('Your database is already up-to-date.')
                exit(0)
            else:
                # set to resume if update to latest is stalled or can resume previous
                self.resume = latest_version.update_stalled or can_resume_previous
        else:
            # set up prompt
            prompt_context = dict(
                latest_version=latest_version,
                previous_version=previous_version,
                can_resume_previous=can_resume_previous,
                since_previous_release=since_previous_release,
            )
            # render prompt string
            prompt = render_to_string(
                'calaccess_raw/updatecalaccessrawdata.txt',
                prompt_context,
            )
            # if the user can resume (latest or previous)
            if latest_version.update_stalled or can_resume_previous:
                # and initially confirms
                if self.confirm_proceed(prompt):
                    # go into resume mode
                    self.resume = True
                # if user does not initially confirm
                else:
                    self.resume = False
                    # if can resume latest (but doesn't) prompt restart
                    if latest_version.update_stalled:
                        confirm_restart = self.confirm_proceed(
                            'Do you want re-start your update?\n'
                        )
                    # if can resume previous (but doesn't) prompt for update to latest
                    elif can_resume_previous:
                        confirm_restart = self.confirm_proceed(
                            'Do you want to update to the latest version?\n'
                        )
                    # cancel if user doesn't confirm restart
                    if not confirm_restart:
                        raise CommandError("Update cancelled")
            # if can't resume
            else:
                self.resume = False
                # cancel if user doesn't initially confirm
                if not self.confirm_proceed(prompt):
                    raise CommandError("Update cancelled")

        # set to force restart if the user could have resumed but didn't
        # or no new version is available
        self.force_restart = (
            (
                latest_version.update_stalled or
                can_resume_previous or
                not created
            ) and not self.resume
        )

        # set the version to be updated
        if not self.force_restart and can_resume_previous:
            self.version = previous_version
        else:
            self.version = latest_version

        if self.verbosity and not self.test_mode:
            if self.resume:
                self.header(
                    "Resuming update to {:%m-%d-%Y %H:%M:%S} snapshot".format(
                        self.version.release_datetime
                    )
                )
            else:
                self.header(
                    "Updating to {:%m-%d-%Y %H:%M:%S} snapshot".format(
                        self.version.release_datetime
                    )
                )

        # if not resuming, reset the update start datetime
        if not self.resume:
            self.version.update_start_datetime = now()
        # if restarting, also erase all the other datetimes
        if self.force_restart:
            self.version.update_finish_datetime = None
            self.version.download_start_datetime = None
            self.version.download_finish_datetime = None
            self.version.extract_start_datetime = None
            self.version.extract_finish_datetime = None

        # save here in case the command doesn't finish
        self.version.save()

        # handle download
        if self.test_mode:
            pass
        # can only download the latest version:
        elif self.version == latest_version:
            # if download completed and not forcing restart
            if self.version.download_completed and not self.force_restart:
                self.log('Already downloaded.')
            # otherwise try downloading
            else:
                self.download()
                if self.verbosity:
                    self.duration()

        # handle file extraction
        if self.test_mode:
            handle_command(TestExtractCommand, verbosity=self.verbosity)
        # if the last extract of the version completed and not forcing restart
        elif self.version.extract_completed and not self.force_restart:
            self.log('Already extracted.')
        else:
            # if the zip isn't there
            if not os.path.exists(self.zip_path):
                # if updating to the lastest
                if self.version == latest_version:
                    self.log(
                        '%s not found. Re-downloading before extracting.' % self.zip_path
                    )
                    self.download()
                    if self.verbosity:
                        self.duration()
                else:
                    raise CommandError(
                        'Incomplete file extraction and %s not found.' % self.zip_path
                    )
            # now extract
            call_command(
                'extractcalaccessrawfiles',
                keep_files=self.keep_files
            )
            if self.verbosity:
                self.duration()

        # refresh the version (to get timestamp field values)
        self.version.refresh_from_db()

        self.clean()
        if self.verbosity:
            self.duration()

        # if archive setting is enabled, zip up all of the csv and error logs
        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            # skip if resuming and the clean zip file is already saved
            if self.resume and bool(self.version.clean_zip_archive):
                self.log('File already zipped.')
            else:
                self.archive()
                if self.verbosity:
                    self.duration()

        self.load()
        if self.verbosity:
            self.duration()

        # store update finish time
        self.version.update_finish_datetime = now()
        # and save the RawDataVersion
        self.version.save()

        if self.verbosity:
            self.success("Done!")

    def download(self):
        """
        Try downloading the zip. Wait and re-try if certain CommandErrors are raised.
        """
        try:
            call_command(
                "downloadcalaccessrawdata",
                verbosity=self.verbosity,
                noinput=True,
                restart=self.force_restart,
            )
        except CommandError as e:
            # if the expected and actual zip size are not the same
            if (
                'expected' in e.message.lower() or
                'modified' in e.message.lower() or
                'version' in e.message.lower()
            ):
                logger.debug('Waiting five minutes before re-trying')
                # wait five minutes
                sleep(300)
                # then try again
                call_command(
                    "downloadcalaccessrawdata",
                    verbosity=self.verbosity,
                    noinput=True,
                    # force a restart on second try
                    restart=True,
                )

    def clean(self):
        """
        Clean up the raw data files from the state so they are ready to get loaded into the database.
        """
        if self.verbosity:
            self.header("Cleaning data files")

        tsv_list = [
            f for f in os.listdir(self.tsv_dir) if '.TSV' in f.upper()
        ]

        if self.resume:
            # get finished clean command logs of last update
            prev_cleaned = [
                x.file_name + '.TSV'
                for x in self.version.files.filter(
                    clean_finish_datetime__isnull=False
                )
            ]
            self.log("{} files already cleaned.".format(len(prev_cleaned)))
            # remove these from tsv_list
            tsv_list = [x for x in tsv_list if x not in prev_cleaned]

        # Loop through all the files in the source directory
        if self.verbosity:
            tsv_list = progress.bar(tsv_list)
        for name in tsv_list:
            call_command(
                "cleancalaccessrawfile",
                name,
                verbosity=self.verbosity,
                keep_file=self.keep_files,
            )

    def archive(self):
        """
        Zip up and archive the .csv and error log files.
        """
        if self.verbosity:
            self.header("Zipping cleaned files")
        # Remove previous zip file
        self.version.clean_zip_archive.delete()
        clean_zip_path = os.path.join(self.data_dir, 'clean.zip')

        # enable zipfile compression
        compression = ZIP_DEFLATED

        try:
            zf = ZipFile(clean_zip_path, 'w', compression, allowZip64=True)
        except RuntimeError:
            logger.error('Zip file cannot be compressed (check zlib module).')
            compression = ZIP_STORED
            zf = ZipFile(clean_zip_path, 'w', compression, allowZip64=True)

        # loop over and save files in csv dir
        for f in os.listdir(self.csv_dir):
            if self.verbosity > 2:
                self.log(" Adding %s to zip" % f)
            csv_path = os.path.join(self.csv_dir, f)
            zf.write(csv_path, f)

        # same for errors dir
        errors_dir = os.path.join(self.data_dir, 'log')
        for f in os.listdir(errors_dir):
            if self.verbosity > 2:
                self.log(" Adding %s to zip" % f)
            error_path = os.path.join(errors_dir, f)
            zf.write(error_path, f)

        # close the zip file
        zf.close()
        if self.verbosity > 2:
            self.log(" All files zipped")

        if not self.test_mode:
            # save the clean zip size
            self.version.clean_zip_size = os.path.getsize(clean_zip_path)
            with open(clean_zip_path, 'rb') as zf:
                # Save the zip on the raw data version
                if self.verbosity > 2:
                    self.log(" Archiving zip")
                self.version.clean_zip_archive.save(
                    os.path.basename(clean_zip_path), File(zf)
                )
            if self.verbosity > 2:
                self.log(" Zip archived.")

    def load(self):
        """
        Loads the cleaned up csv files into the database.
        """
        if self.verbosity:
            self.header("Loading data files")

        model_list = [
            x for x in get_model_list() if os.path.exists(x.objects.get_csv_path())
        ]

        if self.resume:
            # get finished load command logs of last update
            prev_loaded = [
                x.file_name
                for x in self.version.files.filter(
                    load_finish_datetime__isnull=False
                )
            ]
            self.log("{} models already loaded.".format(len(prev_loaded)))
            # remove these from model_list
            model_list = [x for x in model_list if x._meta.db_table not in prev_loaded]

        if self.verbosity:
            model_list = progress.bar(model_list)
        for model in model_list:
            call_command(
                "loadcalaccessrawfile",
                model.__name__,
                verbosity=self.verbosity,
                keep_file=self.keep_files,
                app_name=self.app_name,
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-26 14:49
from __future__ import unicode_literals

import calaccess_raw.annotations
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0004_auto_20160804_1758'),
    ]

    operations = [
        migrations.AlterField(
            model_name='receivedfilingscd',
            name='filing_directory',
            field=calaccess_raw.fields.CharField(db_column='FILING_DIRECTORY', help_text='This field is undocumented', max_length=60),
        ),
        migrations.AlterField(
            model_name='receivedfilingscd',
            name='filing_file_name',
            field=calaccess_raw.fields.CharField(db_column='FILING_FILE_NAME', help_text='The field is undocumented', max_length=60),
        ),
        migrations.AlterField(
            model_name='receivedfilingscd',
            name='form_id',
            field=calaccess_raw.fields.CharField(blank=True, choices=[(b'F400', b'Form 400: Statement of Organization (Slate Mailer Organization)'), (b'F401', b'Form 401: Slate Mailer Organization Campaign Statement'), (b'F402', b'Form 402: Statement of Termination (Slate Mailer Organization)'), (b'F410', b'Form 410: Statement of Organization Recipient Committee'), (b'F425', b'Form 425: Semi-Annual Statement of no Activity'), (b'F450', b'Form 450: Recipient Committee Campaign Disclosure Statement - Short Form'), (b'F460', b'Form 460: Recipient Committee Campaign Statement'), (b'F461', b'Form 461: Independent Expenditure Committee & Major Donor Committee Campaign Statement'), (b'F465', b'Form 465: Supplemental Independent Expenditure Report'), (b'F496', b'Form 496: Late Independent Expenditure Report'), (b'F497', b'Form 497: Late Contribution Report'), (b'F498', b'Form 498: Slate Mailer Late Payment Report'), (b'F601', b'Form 601: Lobbying Firm Registration Statement'), (b'F602', b'Form 602: Lobbying Firm Activity Authorization'), (b'F603', b'Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement'), (b'F604', b'Form 604: Lobbyist Certification Statement'), (b'F606', b'Form 606: Notice of Termination'), (b'F607', b'Form 607: Notice of Withdrawal'), (b'F615', b'Form 615: Lobbyist Report'), (b'F625', b'Form 625: Report of Lobbying Firm'), (b'F635', b'Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition'), (b'F645', b'Form 645: Report of Person Spending $5,000 or More')], db_column='FORM_ID', documentcloud_pages=[calaccess_raw.annotations.DocumentCloud(end_page=8, id='2711624-Overview', start_page=4)], help_text='Form identification code', max_length=7, verbose_name='form identification code'),
        ),
        migrations.AlterField(
            model_name='receivedfilingscd',
            name='receive_comment',
            field=calaccess_raw.fields.CharField(db_column='RECEIVE_COMMENT', help_text='A comment', max_length=120),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for tracking CAL-ACCESS updates over time.
"""
from __future__ import unicode_literals
from .. import managers
from django.db import models
from hurry.filesize import size as sizeformat
from calaccess_raw import archive_directory_path, get_model_list
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class RawDataVersion(models.Model):
    """
    A version of CAL-ACCESS raw source data, typically released every day.
    """
    release_datetime = models.DateTimeField(
        unique=True,
        null=False,
        verbose_name='date and time of release',
        help_text='Date and time the version of the CAL-ACCESS database was released '
                  '(value of last-modified field in HTTP response header)'
    )
    expected_size = models.BigIntegerField(
        null=False,
        verbose_name='expected downloaded zip size',
        help_text='The expected size of the downloaded CAL-ACCESS zip, as '
                  'specified in the content-length field in HTTP response header.'
    )
    update_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time update started',
        help_text='Date and time when the update to the CAL-ACCESS version started',
    )
    update_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time update finished',
        help_text='Date and time when the update to the CAL-ACCESS version finished',
    )
    download_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time download started',
        help_text='Date and time when the download of the CAL-ACCESS database export started',
    )
    download_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time download finished',
        help_text='Date and time when the download of the CAL-ACCESS database export finished',
    )
    extract_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time extraction started',
        help_text='Date and time when extraction of the CAL-ACCESS data files started',
    )
    extract_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time extraction finished',
        help_text='Date and time when extraction of the CAL-ACCESS data files finished',
    )
    download_zip_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='download files zip file',
        help_text='An archive of the original zipped file downloaded from CAL-ACCESS.'
    )
    clean_zip_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='cleaned files zip archive',
        help_text='An archive zip of cleaned (and error log) files'
    )
    download_zip_size = models.BigIntegerField(
        null=True,
        verbose_name='downloaded zip size',
        help_text='The actual size of the downloaded CAL-ACCESS zip after the '
                  'downloaded completed.'
    )
    clean_zip_size = models.BigIntegerField(
        null=True,
        verbose_name='clean zip size',
        help_text='The size of the zip containing all cleaned raw data files '
                  'and error logs.'
    )
    objects = managers.RawDataVersionManager()

    class Meta:
        """
        Meta model options.
        """
        app_label = 'calaccess_raw'
        verbose_name = 'TRACKING: CAL-ACCESS raw data version'
        ordering = ('-release_datetime',)
        get_latest_by = 'release_datetime'

    def __str__(self):
        return str(self.release_datetime)

    @property
    def update_completed(self):
        """
        Check if the database update to the version completed.

        Return True or False.
        """
        if self.update_finish_datetime:
            is_completed = True
        else:
            is_completed = False

        return is_completed

    @property
    def update_stalled(self):
        """
        Check if the database update to the version started but did not complete.

        Return True or False.
        """
        if self.update_start_datetime and not self.update_finish_datetime:
            is_stalled = True
        else:
            is_stalled = False

        return is_stalled

    @property
    def download_completed(self):
        """
        Check if the download of the version's zip completed.

        Return True or False.
        """
        if self.download_finish_datetime:
            is_completed = True
        else:
            is_completed = False

        return is_completed

    @property
    def download_stalled(self):
        """
        Check if the download of the version's zip started but did not complete.

        Return True or False.
        """
        if self.download_start_datetime and not self.download_finish_datetime:
            is_stalled = True
        else:
            is_stalled = False

        return is_stalled

    @property
    def extract_completed(self):
        """
        Check if the extract of files from the downloaded zip completed.

        Return True or False.
        """
        if self.extract_finish_datetime:
            is_completed = True
        else:
            is_completed = False

        return is_completed

    @property
    def extract_stalled(self):
        """
        Check if the extract of files from the downloaded zip started but did not complete.

        Return True or False.
        """
        if self.extract_start_datetime and not self.extract_finish_datetime:
            is_stalled = True
        else:
            is_stalled = False

        return is_stalled

    def pretty_expected_size(self):
        """
        Returns a prettified version (e.g., "725M") of the expected size of the downloaded zip.
        """
        if not self.expected_size:
            return None
        return sizeformat(self.expected_size)
    pretty_expected_size.short_description = 'expected download zip size'
    pretty_expected_size.admin_order_field = 'expected download zip size'

    def pretty_download_size(self):
        """
        Returns a prettified version (e.g., "725M") of the actual size of the downloaded zip.
        """
        if not self.download_zip_size:
            return None
        return sizeformat(self.download_zip_size)
    pretty_download_size.short_description = 'download zip size'
    pretty_download_size.admin_order_field = 'download zip size'

    def pretty_clean_size(self):
        """
        Returns a prettified version (e.g., "725M") of the zip of clean data files and error logs.
        """
        if not self.clean_zip_size:
            return None
        return sizeformat(self.clean_zip_size)
    pretty_clean_size.short_description = 'clean zip size'
    pretty_clean_size.admin_order_field = 'clean zip size'

    @property
    def download_file_count(self):
        """
        Returns the count of files included in the version's downloaded zip.
        """
        return self.files.count()

    @property
    def download_record_count(self):
        """
        Returns the count of records in the version's downloaded files.
        """
        return self.files.aggregate(
            total=models.Sum('download_records_count')
        )['total']

    @property
    def clean_file_count(self):
        """
        Returns the count of files cleaned in the version.
        """
        return self.files.exclude(clean_file_archive='').count()

    @property
    def clean_record_count(self):
        """
        Returns the count of records in the version's cleaned files.
        """
        return self.files.aggregate(
            total=models.Sum('clean_records_count')
        )['total']

    @property
    def error_file_count(self):
        """
        Returns the count of cleaned files with errors in the version.
        """
        return self.files.exclude(error_log_archive='').count()

    @property
    def error_count(self):
        """
        Returns the count of cleaning errors in the version.
        """
        return self.files.aggregate(
            total=models.Sum('error_count')
        )['total']


@python_2_unicode_compatible
class RawDataFile(models.Model):
    """
    A data file included in a version of the CAL-ACCESS raw source data.
    """
    version = models.ForeignKey(
        'RawDataVersion',
        on_delete=models.CASCADE,
        related_name='files',
        verbose_name='raw data version',
        help_text='Foreign key referencing the version of the raw '
                  'source data in which the file was included'
    )
    file_name = models.CharField(
        max_length=100,
        verbose_name='raw data file name',
        help_text='Name of the raw source data file without extension',
    )
    download_records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='download records count',
        help_text='Count of records in the original file downloaded from CAL-ACCESS'
    )
    clean_records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='clean records count',
        help_text='Count of records in the cleaned file generated by calaccess_raw'
    )
    load_records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='load records count',
        help_text="Count of records in the loaded from cleaned file into "
                  "calaccess_raw's data model"
    )
    download_columns_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='download columns count',
        help_text='Count of columns in the original file downloaded from CAL-ACCESS'
    )
    clean_columns_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='clean columns count',
        help_text='Count of columns in the cleaned file generated by calaccess_raw'
    )
    load_columns_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='load columns count',
        help_text='Count of columns on the loaded calaccess_raw data model'
    )
    download_file_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of download file',
        help_text='An archive of the original raw data file downloaded '
                     'from CAL-ACCESS.'
    )
    download_file_size = models.BigIntegerField(
        null=False,
        default=0,
        verbose_name='size of raw data file in bytes',
        help_text='Size of the .TSV file'
    )
    clean_file_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of clean file',
        help_text='An archive of the raw data file after being cleaned.'
    )
    clean_file_size = models.BigIntegerField(
        null=False,
        default=0,
        verbose_name='size of clean data file in bytes',
        help_text='Size of the .CSV file'
    )
    error_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='error count',
        help_text='Count of records in the original download that could not '
                  'be parsed and are excluded from the cleaned file.'
    )
    error_log_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of error log',
        help_text='An archive of the error log containing lines from the '
                  'original download file that could not be parsed and are '
                  'excluded from the cleaned file.'
    )
    clean_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time cleaning started',
        help_text='Date and time when the cleaning of the file started',
    )
    clean_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time cleaning finished',
        help_text='Date and time when the cleaning of the file finished',
    )
    load_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time loading started',
        help_text='Date and time when the loading of the file started',
    )
    load_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time extraction finished',
        help_text='Date and time when the loading of the file finished',
    )

    class Meta:
        """
        Meta model options.
        """
        app_label = 'calaccess_raw'
        unique_together = (('version', 'file_name'),)
        verbose_name = 'TRACKING: CAL-ACCESS raw data file'
        ordering = ('-version_id', 'file_name',)

    def __str__(self):
        return self.file_name

    def pretty_download_file_size(self):
        """
        Returns a prettified version (e.g., "725M") of the downloaded file's size.
        """
        return sizeformat(self.download_file_size)
    pretty_download_file_size.short_description = 'download file size'
    pretty_download_file_size.admin_order_field = 'download file size'

    def pretty_clean_file_size(self):
        """
        Returns a prettified version (e.g., "725M") of the cleaned file's size.
        """
        return sizeformat(self.clean_file_size)
    pretty_clean_file_size.short_description = 'clean file size'
    pretty_clean_file_size.admin_order_field = 'clean file size'

    @property
    def model(self):
        """
        Returns the RawDataFile's corresponding CalAccess database model object.
        """
        try:
            return [
                m for m in get_model_list() if m().db_table == self.file_name
            ][0]
        except IndexError:
            return None



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-31 01:32
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0006_auto_20160826_2039'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filernamecd',
            name='naml',
            field=calaccess_raw.fields.CharField(db_column='NAML', help_text='Last name, sometimes full name in the case of PACs, firms and employers. Major donors can be split between first and last name fields, but usually are contained in the last name field only. Individual names of lobbyists, politicans and officers tend to use both the first and last name.', max_length=200),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-26 20:39
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0005_auto_20160826_1449'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rcptcd',
            name='amount',
            field=calaccess_raw.fields.DecimalField(db_column='AMOUNT', decimal_places=2, help_text='Amount Received (Monetary, In-kind, Promise)', max_digits=14),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-04 14:43
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0002_auto_20160802_2101'),
    ]

    operations = [
        migrations.RenameField('RawDataVersion', 'size', 'expected_size'),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-26 16:43
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0009_auto_20161026_1641'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cvr2campaigndisclosurecd',
            name='cmte_id',
            field=calaccess_raw.fields.CharField(blank=True, db_column='CMTE_ID', help_text='Committee identification number, when the entity is a committee', max_length=9),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Extract the CAL-ACCESS raw data files from downloaded ZIP.
"""
import os
import shutil
import zipfile
from django.conf import settings
from django.core.files import File
from django.core.management.base import CommandError
from django.utils.timezone import now
from calaccess_raw import get_download_directory, get_test_download_directory
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models.tracking import RawDataVersion, RawDataFile


class Command(CalAccessCommand):
    """
    Extract the CAL-ACCESS raw data files from downloaded ZIP.
    """
    help = "Extract the CAL-ACCESS raw data files from downloaded ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep downloaded zipped files"
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # get the dir where data goes from app settings
        self.data_dir = get_download_directory()

        # get path to download zip file
        self.zip_path = os.path.join(self.data_dir, self.url.split('/')[-1])
        # raw tsv files go in same data_dir in tsv/
        self.tsv_dir = os.path.join(self.data_dir, "tsv/")

        # get most recently downloaded RawDataVersion
        try:
            self.version = RawDataVersion.objects.filter(
                download_start_datetime__isnull=False
            ).latest('download_start_datetime')
        except RawDataVersion.DoesNotExist:
            raise CommandError(
                'No record of download CAL-ACCESS zip file '
                '(run `python manage.py downloadcalaccessrawdata`).'
            )

        # raise exception if extract step did not finish
        if not self.version.download_completed:
            raise CommandError(
                'Previous download did not finish '
                '(run `python manage.py downloadcalaccessrawdata`).'
            )

        # store extraction start time
        self.version.extract_start_datetime = now()
        # and reset the finish time
        self.version.extract_finish_datetime = None
        # save here in case the command doesn't finish
        self.version.save()

        self.header("Extracting raw data files")

        self.unzip()
        self.prep()
        self.track_files()

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.archive()

        if not options['keep_files']:
            os.remove(self.zip_path)
            shutil.rmtree(os.path.join(self.data_dir, 'CalAccess'))

        # store extraction finish time
        self.version.extract_finish_datetime = now()
        # and save the RawDataVersion
        self.version.save()

    def unzip(self):
        """
        Unzip the snapshot file.
        """
        if self.verbosity:
            self.log(" Unzipping %s" % os.path.basename(self.zip_path))

        with zipfile.ZipFile(self.zip_path) as zf:
            for member in zf.infolist():
                words = member.filename.split('/')
                path = self.data_dir
                for word in words[:-1]:
                    drive, word = os.path.splitdrive(word)
                    head, word = os.path.split(word)
                    if word in (os.curdir, os.pardir, ''):
                        continue
                    path = os.path.join(path, word)
                zf.extract(member, path)

    def prep(self):
        """
        Rearrange the unzipped files and get rid of the stuff we don't want.
        """
        if self.verbosity:
            self.log(" Prepping unzipped data")

        # Move the deep down directory we want out
        shutil.move(
            os.path.join(
                self.data_dir,
                'CalAccess/DATA/CalAccess/DATA/'
            ),
            self.data_dir
        )
        # Clear out target if it exists
        if os.path.exists(self.tsv_dir):
            shutil.rmtree(self.tsv_dir)

        # Rename it to the target
        shutil.move(
            os.path.join(self.data_dir, "DATA/"),
            self.tsv_dir,
        )

    def track_files(self):
        """
        Create a RawDataFile for each download .TSV file.
        """
        for f in os.listdir(self.tsv_dir):
            if '.TSV' in f:
                file_name = f.upper().replace('.TSV', '')
                raw_file, created = RawDataFile.objects.get_or_create(
                    version=self.version,
                    file_name=file_name,
                )
                # if raw file was already there, clear out timestamp fields
                if not created:
                    raw_file.clean_start_datetime = None
                    raw_file.clean_finish_datetime = None
                    raw_file.load_start_datetime = None
                    raw_file.load_finish_datetime = None
                    raw_file.save()

    def archive(self):
        """
        Save a copy of the download zip file and each file inside.
        """
        if self.verbosity:
            self.log(" Archiving original files")
        if self.verbosity > 2:
            self.log(" Archiving {0}".format(os.path.basename(self.zip_path)))

        for raw_file in self.version.files.all():
            if self.verbosity > 2:
                self.log(" Archiving {0}.TSV".format(raw_file.file_name))
            # Remove previous .TSV file
            raw_file.download_file_archive.delete()
            # Open up the .TSV file so we can wrap it in the Django File obj
            with open(self.tsv_dir + raw_file.file_name + '.TSV', 'rb') as f:
                # Save the .TSV on the raw data file
                raw_file.download_file_archive.save(
                    raw_file.file_name + '.TSV',
                    File(f)
                )


class TestCommand(Command):
    """
    Simulates the unzipping and preping of CAL-ACCESS raw data for testing.
    """
    help = "Simulates the unzipping and preping of CAL-ACCESS raw data for testing"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(TestCommand, self).add_arguments(parser)

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        self.verbosity = options.get("verbosity")
        self.no_color = options.get("no_color")
        self.data_dir = get_test_download_directory()
        self.tsv_dir = os.path.join(self.data_dir, "tsv/")
        self.zip_path = os.path.join(self.data_dir, self.url.split('/')[-1])

        with open(self.data_dir + "/sampled_version.txt", "r") as f:
            length = f.readline()
            release_datetime = f.readline()

        try:
            # long int type is big enough for double the current size of the zip
            expected_size = long(length)
        except NameError:
            # in py3, no long(), instead int will suffice
            expected_size = int(length)

        # get or create the RawDataVersion
        self.version, created = self.get_or_create_version(
            expected_size,
            self.parse_imf_datetime_str(
                release_datetime,
            )
        )

        # store extraction start time
        self.version.extract_start_datetime = now()
        # and save the RawDataVersion
        self.version.save()

        self.unzip()
        self.prep()
        self.track_files()

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.archive()

        # store extraction finish time
        self.version.extract_finish_datetime = now()
        # and save the RawDataVersion
        self.version.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-04 17:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0003_auto_20160804_1443'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawdataversion',
            name='clean_zip_size',
            field=models.BigIntegerField(help_text='The size of the zip containing all cleaned raw data files and error logs.', null=True, verbose_name='clean zip size'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='download_zip_size',
            field=models.BigIntegerField(help_text='The actual size of the downloaded CAL-ACCESS zip after the downloaded completed.', null=True, verbose_name='downloaded zip size'),
        ),
        migrations.AlterField(
            model_name='rawdataversion',
            name='expected_size',
            field=models.BigIntegerField(help_text='The expected size of the downloaded CAL-ACCESS zip, as specified in the content-length field in HTTP response header.', verbose_name='expected downloaded zip size'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup


setup(
    name='django-calaccess-raw-data',
    version='1.5.2',
    packages=(
        'calaccess_raw',
        'calaccess_raw.admin',
        'calaccess_raw.management',
        'calaccess_raw.migrations',
        'calaccess_raw.management.commands',
        'calaccess_raw.models',
        'calaccess_raw.annotations',
    ),
    include_package_data=True,
    license='MIT',
    description="A Django app to download, extract and load campaign "
                "finance and lobbying activity data from the California "
                "Secretary of State\'s CAL-ACCESS database",
    url='http://django-calaccess.californiacivicdata.org/',
    author='California Civic Data Coalition',
    author_email='cacivicdata@gmail.com',
    install_requires=(
        'django>=1.8',
        'django-postgres-copy>=0.1.0',
        'csvkit>=1.0',
        'requests',
        'clint',
        'hurry.filesize',
        'pytz',
    ),
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'License :: OSI Approved :: MIT License',
    ),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utilities for representing and interacting with CAL-ACCESS reference documents and forms.
"""
import os
import json
import requests
from django.conf import settings
from calaccess_raw import get_model_list
from django.utils.deconstruct import deconstructible


@deconstructible
class DocumentCloud(object):
    """
    A document hosted on DocumentCloud.

    Cited in our Python code and then republished in our HTML documentation.
    """
    def __init__(self, id, start_page=None, end_page=None):
        """
        Create a new object by submitting a unique ID from DocumentCloud.org.
        """
        self.id = id
        self.start_page = start_page
        self.end_page = end_page
        self.metadata_cache_dir = os.path.join(
            settings.BASE_DIR,
            ".documentcloud_metadata"
        )
        self.metadata_filename = os.path.join(
            self.metadata_cache_dir,
            '{id}.json'.format(id=self.id)
        )

    def _request_metadata(self):
        """
        Returns contents of GET request to /api/documents/[id].json method.
        """
        r = requests.get(
            'https://www.documentcloud.org/documents/{id}.json'.format(id=self.id)
        )
        return r.content.decode('utf-8')

    def _cache_metadata(self):
        """
        Requests metadata and stores in .json file in .documentcloud_cache dir.

        Creates .documentcloud_cache dir if it doesn't already exist.
        """
        os.path.exists(self.metadata_cache_dir) or os.makedirs(self.metadata_cache_dir)

        with open(self.metadata_filename, 'w') as f:
            f.write(self._request_metadata())

    @property
    def metadata(self):
        """
        Returns a dictionary with the document's metadata retrieved from DocumentCould.
        """
        if not os.path.exists(self.metadata_filename):
            self._cache_metadata()

        with open(self.metadata_filename) as f:
            self._metadata = json.loads(f.read())

        return self._metadata

    @property
    def title(self):
        """
        Returns the title of the document.
        """
        self._title = self.metadata['title']
        return self._title

    @property
    def canonical_url(self):
        """
        Returns the URL where the document can be found on DocumentCloud.
        """
        if self.start_page:
            canonical_url = (
                self.metadata['canonical_url'] +
                '#document/p{}'.format(self.start_page)
            )
        else:
            canonical_url = self.metadata['canonical_url']
        return canonical_url

    @property
    def thumbnail_url(self):
        """
        Returns a URL to the thumbnail image of the document's first page.
        """
        page = self.start_page or 1
        self._thumbnail_url = self.metadata['resources']['page']['image'].format(
            size='thumbnail',
            page=page
        )
        return self._thumbnail_url

    @property
    def pdf_url(self):
        """
        Returns a URL to the full PDF of the document.
        """
        self._pdf_url = self.metadata['resources']['pdf']
        return self._pdf_url

    @property
    def text_url(self):
        """
        Returns a URL to the full text of the document.
        """
        self._text_url = self.metadata['resources']['text']
        return self._text_url

    @property
    def num_pages(self):
        """
        Returns the number of pages in this document.
        """
        if self.start_page and self.end_page:
            self._num_pages = self.end_page - self.start_page + 1
        elif self.end_page:
            self._num_pages = self.end_page
        elif self.start_page:
            self._num_pages = 1
        # ignored case: User wants to specify and start page and
        # expects to include all subsequent pages in doc
        else:
            self._num_pages = self.metadata['pages']

        return self._num_pages

    @property
    def pages(self):
        """
        Returns a list of the pages in this form as DocPage objects.
        """
        class DocPage(object):
            def __init__(self, num, canonical_url, thumbnail_url):
                self.num = num
                self.canonical_url = canonical_url
                self.thumbnail_url = thumbnail_url

        canonical_url = 'https://www.documentcloud.org/documents/{id}/pages/{page}.html'

        image_url = self.metadata['resources']['page']['image']

        start = self.start_page or 1

        return [
            DocPage(
                x,
                canonical_url.format(id=self.id, page=x),
                image_url.format(size='thumbnail', page=x)
            ) for x in range(start, start + self.num_pages)
        ]

    @property
    def formatted_page_nums(self):
        """
        Returns the page range as a pretty string.
        """
        if self.end_page:
            formatted_str = '{0}-{1}'.format(self.start_page, self.end_page)
        else:
            formatted_str = str(self.start_page)
        return formatted_str


@deconstructible
class FilingForm(object):
    """
    A form used to collect information for the CAL-ACCESS database.
    """
    def __init__(self, id, title, **kwargs):
        """
        Create a new object by submitting a unique ID and title.
        """
        self.id = id
        self.title = title
        self.description = kwargs.get('description')
        self.group = kwargs.get('group')
        self.documentcloud_id = kwargs.get('documentcloud_id')
        self.db_value = kwargs.get('db_value', self.id)
        self.sections = []
        if self.documentcloud_id:
            self.documentcloud = DocumentCloud(self.documentcloud_id)
        else:
            self.documentcloud = None

    @property
    def type_and_num(self):
        """
        Returns a short title for the form that includes its type and number.
        """
        if self.id[0] == 'E':
            self._type_and_num = 'Electronic Form {}'.format(self.id[1:])
        elif self.id[0] == 'S':
            self._type_and_num = 'Schedule {}'.format(self.id[1:])
        else:
            self._type_and_num = 'Form {}'.format(self.id[1:])

        return self._type_and_num

    @property
    def full_title(self):
        """
        Returns the full title of the form.
        """
        self._full_title = '{0}: {1}'.format(self.type_and_num, self.title)
        return self._full_title

    def add_section(self, id, title, **kwargs):
        """
        Adds a Section with the provided title and options to this object.
        """
        new_section = FilingFormSection(
            form=self,
            id=id,
            title=title,
            db_value=kwargs.get('db_value', self.db_value),
            start_page=kwargs.get('start_page'),
            end_page=kwargs.get('end_page'),
            documentcloud_id=kwargs.get('documentcloud_id'),
        )
        self.sections.append(new_section)
        return new_section

    def get_section(self, id):
        """
        Returns the Section object with the given id.
        """
        section_dict = {i.id: i for i in self.sections}
        return section_dict[id]

    def get_models(self):
        """
        Returns all the CAL-ACCESS models connected with this form.
        """
        models = []
        for model in get_model_list():
            if self in [x[0] for x in model().get_filing_forms_w_sections()]:
                models.append(model)

        return models

    def __str__(self):
        return str(self.id)


@deconstructible
class FilingFormSection(object):
    """
    A section of a FilingForm (e.g., a cover page, summary sheet, schedule or part).
    """
    def __init__(self, form, id, title, **kwargs):
        """
        Create a new object by submitting a FilingForm parent with an ID and title.
        """
        self.form = form
        self.id = id
        self.title = title
        self.db_value = kwargs.get('db_value', form.db_value)
        self.start_page = kwargs.get('start_page')
        self.end_page = kwargs.get('end_page')
        self.documentcloud = DocumentCloud(
            self.form.documentcloud_id,
            self.start_page,
            self.end_page
        )

    @property
    def full_title(self):
        """
        Returns full title of the section, including the parent form's name.
        """
        self._full_title = '{0} ({1}): {2}'.format(
            self.form.type_and_num,
            self.form.title,
            self.title,
        )
        return self._full_title

    def __str__(self):
        return str(self.id)


def get_sorted_choices(codes_dict):
    """
    Returns a tuple of tuples, sorted by the given codes_dict's key.
    """
    return tuple(sorted(codes_dict.items(), key=lambda x: x[0]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A hodgepodge of utilities related to the app's settings and configuration.
"""
import os
from django.conf import settings
default_app_config = 'calaccess_raw.apps.CalAccessRawConfig'


def get_download_directory():
    """
    Returns download directory for data storage downloaded data.
    """
    if getattr(settings, 'CALACCESS_DOWNLOAD_DIR', None):
        return getattr(settings, 'CALACCESS_DOWNLOAD_DIR')
    elif getattr(settings, 'BASE_DIR', None):
        return os.path.join(getattr(settings, 'BASE_DIR'), 'data')
    raise ValueError("CAL-ACCESS download directory not configured. Set either \
CALACCESS_DOWNLOAD_DIR or BASE_DIR in settings.py")


def get_test_download_directory():
    """
    Returns download directory where we will store test data.
    """
    if getattr(settings, 'CALACCESS_TEST_DOWNLOAD_DIR', None):
        return getattr(settings, 'CALACCESS_TEST_DOWNLOAD_DIR')
    elif getattr(settings, 'BASE_DIR', None):
        return os.path.join(getattr(settings, 'BASE_DIR'), 'test-data')
    raise ValueError("CAL-ACCESS test download directory not configured. \
Set either CALACCESS_TEST_DOWNLOAD_DIR or BASE_DIR in settings.py")


def archive_directory_path(instance, filename):
    """
    Returns a path to an archived raw data file or ZIP.

    (e.g., MEDIA_ROOT/YYYY-MM-DD_HH-MM-SS/filename.ext)
    """
    from calaccess_raw.models.tracking import RawDataVersion, RawDataFile

    if isinstance(instance, RawDataVersion):
        release_datetime = instance.release_datetime
        f_name, f_ext = filename.split('.')
        path = '{fn}_{dt:%Y-%m-%d_%H-%M-%S}.{fx}'.format(
            fn=f_name,
            dt=release_datetime,
            fx=f_ext,
        )
    elif isinstance(instance, RawDataFile):
        release_datetime = instance.version.release_datetime
        path = '{dt:%Y-%m-%d_%H-%M-%S}/{f}'.format(dt=release_datetime, f=filename)
    else:
        raise TypeError("Must be called on RawDataVersion or RawDataFile instance.")
    return path


def get_model_list():
    """
    Returns a model list with all the data tables in this application.
    """
    from django.apps import apps
    model_list = apps.get_app_config("calaccess_raw").models.values()
    return [m for m in model_list if "CalAccessBaseModel" in str(m.__base__)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Generate report outlining the number / proportion of files / records cleaned and loaded.
"""
from __future__ import unicode_literals
from __future__ import division
import os
from csv import DictWriter
from clint.textui import progress
from django.core.management import call_command
from django.core.management.base import CommandError
from django.db.models import Sum
from django.forms.models import model_to_dict
from calaccess_raw import get_model_list, get_download_directory
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models.tracking import RawDataVersion


def calc_percent(whole, total):
    """
    Calculate percentage of the total.
    """
    try:
        pct = whole / total
    except ZeroDivisionError:
        pct = 0
    else:
        pct * 100

    return pct


class Command(CalAccessCommand):
    """
    Generate report outlining the number / proportion of files / records cleaned and loaded.
    """
    help = 'Generate report outlining the number / proportion of files / records cleaned and loaded'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        self.missing_raw_files = []
        self.empty_raw_files = []
        self.unknown_raw_files = []

        # get the latest version
        try:
            self.version = RawDataVersion.objects.latest('release_datetime')
        except RawDataVersion.DoesNotExist:
            raise CommandError('No CAL-ACCESS versions tracked.')
        # quit if version update not complete
        if not self.version.update_completed:
            raise CommandError(
                'Update to {:%c} version is not complete. (run `python manage.py '
                'updatecalaccessrawdata`).'.format(self.version.release_datetime)
            )

        self.log("Analyzing loaded models")

        if self.verbosity == 1:
            model_list = progress.bar(get_model_list())
        else:
            model_list = get_model_list()

        for model in model_list:

            call_command(
                'verifycalaccessrawfile',
                model.__name__,
                verbosity=self.verbosity
            )

        self.num_download_files = self.version.files.count()

        self.num_clean_files = self.version.files.filter(
            clean_records_count__gt=0
        ).count()

        self.num_loaded_files = self.version.files.filter(
            load_records_count__gt=0
        ).count()

        total_raw_records = self.sum_count_column('download_records_count')

        total_clean_records = self.sum_count_column('clean_records_count')

        total_loaded_records = self.sum_count_column('load_records_count')

        self.log(
            '{0:.4%} of records cleaned'.format(
                calc_percent(total_clean_records, total_raw_records)
            )
        )

        self.log(
            '{0:.4%} of records loaded'.format(
                calc_percent(total_loaded_records, total_raw_records)
            )
        )

        self.write_csv_results()

        self.duration()

    def sum_count_column(self, column_name):
        """
        Takes the name of a RawDataFile count column. Returns a sum of its values.
        """
        result = self.version.files.aggregate(
            the_sum=Sum(column_name)
        )
        return result['the_sum']

    def write_csv_results(self):
        """
        Write .csv file to the docs directory.
        """
        file_name = os.path.join(
            get_download_directory(),
            'calaccess_raw_files_report.csv'
        )

        with open(file_name, 'w') as f:
            fieldnames = [
                'file_name',
                'download_columns_count',
                'clean_columns_count',
                'load_columns_count',
                'download_records_count',
                'clean_records_count',
                'load_records_count',
                'error_count',
                'pct_cleaned',
                'pct_loaded'
            ]
            writer = DictWriter(f, fieldnames=fieldnames)
            writer.writeheader()

            for i in self.version.files.all():

                row = model_to_dict(i)

                for k in row.keys():
                    if k not in fieldnames:
                        del row[k]

                row.update({
                    'pct_cleaned': '{0:.2%}'.format(
                        calc_percent(i.clean_records_count, i.download_records_count)
                    )
                })

                row.update({
                    'pct_loaded': '{0:.2%}'.format(
                        calc_percent(i.load_records_count, i.download_records_count)
                    )
                })

                writer.writerow(row)
